unit UCadArquivo;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Grids, DBGrids, StdCtrls, DBCtrls,
  Buttons, DB, ShellAPI, FileCtrl, Jpeg;

type
  TfrmCadArquivo = class(TForm)
    p1: TPanel;
    bbAddArq: TBitBtn;
    bbDelArq: TBitBtn;
    lstArquivos: TListBox;
    pBotoes: TPanel;
    bbImportar: TBitBtn;
    p2: TPanel;
    bbAbrir: TBitBtn;
    bbDelTabela: TBitBtn;
    bbSalvar: TBitBtn;
    bbSalvarTodos: TBitBtn;
    p4: TPanel;
    gbx1: TGroupBox;
    p3: TPanel;
    bbObs: TBitBtn;
    memObs: TDBMemo;
    Splitter1: TSplitter;
    gbx2: TGroupBox;
    img: TImage;
    bbFechar: TBitBtn;
    SaveDialog: TSaveDialog;
    DBGrid1: TDBGrid;
    dsArqruivo: TDataSource;
    procedure bbAddArqClick(Sender: TObject);
    procedure bbDelArqClick(Sender: TObject);
    procedure bbFecharClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure bbImportarClick(Sender: TObject);
    procedure memObsKeyPress(Sender: TObject; var Key: Char);
    procedure bbObsClick(Sender: TObject);
    procedure dsArqruivoDataChange(Sender: TObject; Field: TField);
    procedure dsArqruivoStateChange(Sender: TObject);
    procedure bbAbrirClick(Sender: TObject);
    procedure bbDelTabelaClick(Sender: TObject);
    procedure bbSalvarClick(Sender: TObject);
  private
    { Private declarations }
    FPessoa : Integer;  // armazena o cdigo da pessoa que ser passada por parmetros
    FTemp : String;   // armazena a localizao da pasta temp do windows

    Procedure Gravar;
    procedure Status;
    Procedure Copia(AOrigem, ADestino : String);
  public
    { Public declarations }
    // Class Procedure para chamar a janela
    Class Procedure Executar(APessoa : Integer);
  end;

Const
  // "\\<NOME DO COMPUTADOR>\<PASTA COMPARTILHADA$>\"
  COMPARTILHAMENTO = '\\GEORGE\ANEXOS$\';

var
  frmCadArquivo: TfrmCadArquivo;

implementation

uses uDM;

{$R *.dfm}

procedure TfrmCadArquivo.bbAbrirClick(Sender: TObject);
Var
  sOrigem, sDestino : String;
begin
  // copio os arquivos para a pasta temp e abro
  sOrigem  := COMPARTILHAMENTO+trim(dm.cdsArquivoNOME_ARQUIVO.AsString);
  sDestino := FTemp+trim(dm.cdsArquivoNOME_ARQUIVO.AsString);
  // copia o arquivo original para a pasta temp
  Copia(sOrigem,sDestino);
  if (fileExists(sDestino)) then
     // incluir a unit ShellAPI no Uses para utilizar a funo ShellExecute
     ShellExecute(Handle, nil, Pchar(sDestino), nil, nil, SW_SHOWNORMAL)
     // a funo ShellExecute abrir o arquivo com o programa associado
     // pelo Windows
  Else
  Begin
    MessageDlg('Falha ao tentar abrir o arquivo.', mtWarning, [mbOK], 0);
  End;
end;

procedure TfrmCadArquivo.bbAddArqClick(Sender: TObject);
// evento para adicionar os arquivos no ListBox
Var
  sArquivo : String;
begin
  // abre a janela para escolher os arquivos
  If SaveDialog.Execute Then
  Begin
    // guardo o path + nome do arquivo
    sArquivo := SaveDialog.FileName;
    // adiciona no ListBox
    lstArquivos.Items.Add(sArquivo);
    // habilita o boto "Retirar da Seleo"
    bbDelArq.Enabled   := True;
    // habilita o boto "Importar"
    bbImportar.Enabled := True;
  End;
end;


procedure TfrmCadArquivo.bbDelArqClick(Sender: TObject);
// deleta a linha do ListBox
begin
  // se o list box estiver vazio
  If lstArquivos.ItemIndex = -1 Then
    Exit; // sai e no faz nada

  // apaga a linha do ListBox
  lstArquivos.Items.Delete(lstArquivos.ItemIndex);
  // se o ListBox fica vazio aps a excluso...
  If lstArquivos.Items.Count = 0 Then
  Begin
    // desabilito os botes "Importar" e
    // "Retirar da seleo"
    bbImportar.Enabled  := False;
    bbDelArq.Enabled    := False;
  End;
end;

procedure TfrmCadArquivo.bbDelTabelaClick(Sender: TObject);
  procedure ApagarArq; // procedure local do evento OnClick
  // procedimento para apagar o arquivo da pasta compartilhada
  Var
    sOrigem : String;
  begin
    Try
      sOrigem  := COMPARTILHAMENTO+dm.cdsArquivoNOME_ARQUIVO.AsString;
      If not FileExists(sOrigem) Then
      // verifica se o arquivo existe na pasta compartilhada
      Begin
        Raise Exception.Create('Arquivo no encontrado.');
      End;
      If not DeleteFile(sOrigem) Then
      // se der erro ao apagar o arquivo
      Begin
        Raise Exception.Create('DeleteFile error.');
      End;
      // apaga o registro da tabela
      dm.cdsArquivo.Delete;
      // atualiza o banco de dados
      If dm.cdsArquivo.ApplyUpdates(0) > 0 Then
      Begin
        Raise Exception.Create('Erro ApplyUpdates');
      End;
      MessageDlg('Arquivo apagado.', mtInformation, [mbOK], 0);
    Except
      On E : Exception Do
      Begin
        // se der erro
        dm.cdsArquivo.CancelUpdates;
        MessageDlg(Format('Erro ao apagar o arquivo %s.',[sOrigem])+#13+#10+
                   'Erro: '+e.Message, mtWarning, [mbOK], 0);
      End;
    End;
  end;

begin
  if (MessageDlg('Confirma a excluso do arquivo selecionado?', mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
  Begin
    ApagarArq;
  End;
end;

procedure TfrmCadArquivo.bbFecharClick(Sender: TObject);
begin
  dm.cdsArquivo.Close;
  Close;
end;

procedure TfrmCadArquivo.bbImportarClick(Sender: TObject);
begin
  if (MessageDlg('Confirma o(s) arquivos(s) selecionado(s)?', mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
  Begin
    Gravar;
  End;
end;

procedure TfrmCadArquivo.bbObsClick(Sender: TObject);
begin
  bbObs.Enabled := False;
  dm.cdsArquivo.ApplyUpdates(0);
end;

procedure TfrmCadArquivo.bbSalvarClick(Sender: TObject);
// este evento  utilizado para os botes "Salvar para..."
// e "Salvar Todos para..."
Var
  sPasta,
  sOrigem,
  sDestino,
  sErro : String;
  iTotErro : Integer;
  bSalvaTodos : Boolean;
begin
  sErro    := '';
  iTotErro := 0;
  If Sender = bbSalvar Then
  // copia um arquivo da pasta compartilhada para a pasta selecionada
    bSalvaTodos := False
  Else If Sender = bbSalvarTodos Then
  // copia todos os arquivos da pasta compartilhada para a pasta selecionada
    bSalvaTodos := True;

  // Adicione FileCtrl na clusula Uses
  If SelectDirectory('Selecione a pasta',  // caption da janela
                     '',  // Root
                     sPasta,  // pasta que sero copiados os arquivos
                     [sdNewFolder,sdShowEdit,sdShowFiles, sdNewUI,sdShowShares,sdValidateDir]) // opes
                     Then
  // Consulte no Help do Delphi mais informaes sobre a funo SelectDirectory
  Begin
    If bSalvaTodos Then // se for para salva todos...
    Begin
      dm.cdsArquivo.First;  // ...vai para o inicio do cds
    End;
    While not dm.cdsArquivo.Eof Do
    Begin
      sOrigem  := COMPARTILHAMENTO+trim(dm.cdsArquivoNOME_ARQUIVO.AsString);
      // a funo "IncludeTrailingPathDelimiter" inclui uma barra "/" no final,
      // caso seja necessrio
      sDestino := IncludeTrailingPathDelimiter(sPasta)+
                  trim(dm.cdsArquivoNOME_ORIGINAL.AsString);
      Try
        Copia(sOrigem,sDestino);
      Except
        On E : Exception Do
        Begin
          Inc(iTotErro);
          sErro := sErro + 'Ficheiro: '+dm.cdsArquivoNOME_ORIGINAL.AsString+' - '+e.Message;
        End;
      End;
      If bSalvaTodos Then
        dm.cdsArquivo.Next
      Else
        // se for apenas um arquivo, saio do loop
        Break;
    End;
    // verifico os erros
    If iTotErro > 0 Then
    Begin
      MessageDlg('Falha ao copiar o(s) arquivo(s).'+
                 #13+#10+'Erro:'+sErro, mtWarning, [mbOK], 0);
    End
    Else
    Begin
      MessageDlg('Arquivo(s) copiado(s) com sucesso!', mtInformation, [mbOK], 0);
    End;
  End;
end;

procedure TfrmCadArquivo.dsArqruivoDataChange(Sender: TObject; Field: TField);
Var
  sArquivo : String;
begin
  Try
    // retorna o nome do arquivo da pasta compartilhada
    sArquivo := COMPARTILHAMENTO+trim(dm.cdsArquivoNOME_ARQUIVO.AsString);
    // se o arquivo for do tipo imagem, ser exibido no TImage
    // Incluir a unit Jpeg no Uses para ler arquivos do tipo JPG
    img.Picture.LoadFromFile(sArquivo);
  Except
    // se no for do tipo imagem, o erro ser tratado para no exibir para o
    // usurio
    img.Picture := nil;
  End;
  // chama a procedure de status do DataSource
  Status;
end;

procedure TfrmCadArquivo.dsArqruivoStateChange(Sender: TObject);
begin
  // chama a procedure de status do DataSource
  Status;
end;

class procedure TfrmCadArquivo.Executar(APessoa: Integer);
// class procedure para chamada desta janela
begin
  Try
    frmCadArquivo := TfrmCadArquivo.Create(nil);
    // cdigo da pessoa passada por parametros
    frmCadArquivo.FPessoa := APessoa;
    // mostra a janeja
    frmCadArquivo.ShowModal;
  Finally
    FreeAndNil(frmCadArquivo);
  End;
end;

procedure TfrmCadArquivo.FormShow(Sender: TObject);
begin
  // abre a tabela com os registros da pessoa
  dm.cdsArquivo.Close;
  dm.cdsArquivo.Params.ParamByName('ID_PESSOA').AsInteger := FPessoa;
  dm.cdsArquivo.Open;

  // retorna a pasta temp do Windows
  FTemp := IncludeTrailingPathDelimiter(GetEnvironmentVariable('temp'));
end;

procedure TfrmCadArquivo.Gravar;
Var
  i, iTotErro : Integer;
  sExt, sTXTErro, sErro, sOrigem, sDestino : String;

  Function CopiaArquivo(AOrigem, ADestino : String; var AErro : String) : Boolean;
  // copia o arquivo
  begin
    Result := True;
    // verifica se o arq existe
    If not FileExists(AOrigem) Then
    Begin
      Result := False;
      AErro := 'O arquivo no existe no sistema.';
    End
    // copia o arquivo
    Else If not CopyFile(PChar(AOrigem),PChar(ADestino),false) Then
    Begin
      Result := False;
      AErro := 'Falha ao copiar o arquivo.';
    End;
  end;


begin
  iTotErro := 0;
  sTXTErro := '';
  Repeat
    Try
      i := 0;
      // nome original do arquivo
      sOrigem  := lstArquivos.Items.Strings[i];

      dm.cdsArquivo.Append;
      dm.cdsArquivoID_PESSOA.AsInteger    := FPessoa;
      dm.cdsArquivoDATA_CRIACAO.AsDateTime:= date;
      dm.cdsArquivoNOME_ORIGINAL.AsString := ExtractFileName(sOrigem);
      // como o campo NOME_ARQUIVO  obrigatrio,
      // gravo a string TMP para depois gravar o nome corretamente
      dm.cdsArquivoNOME_ARQUIVO.AsString  := 'TMP';
      If dm.cdsArquivo.ApplyUpdates(0) > 0 Then
      Begin
        Raise Exception.Create('Falha ao criar o registro');
      End;
      // atualizo o ClientDataSet para pegar o ID do arquivo
      dm.cdsArquivo.Close;
      dm.cdsArquivo.Open;
      dm.cdsArquivo.Last; // ltimo registro

      // somente o nome original do arquivo+extenso
      sDestino := ExtractFileName(sOrigem);
      // somente a extenso do arquivo original
      sExt     := ExtractFileExt(sOrigem);
      // o nome do arquivo de destino ser o ID da tabela Arquivo +
      // a extenso do nome do arquivo original
      sDestino := trim(IntToStr(dm.cdsArquivoID_ARQUIVO.AsInteger))+sExt;
      // o arq de destino ter o nome da pasta compartilhada +
      // o arquivo renomeado
      sDestino := COMPARTILHAMENTO+sDestino;

      // copio o arquivo para a pasta compartilhada
      If not CopiaArquivo(sOrigem,sDestino,sErro) Then
      Begin
        // se ocorram erros ao copiar o arquivo
        // apago o registro criado
        dm.cdsArquivo.Delete;
        dm.cdsArquivo.ApplyUpdates(0);
        // retorno uma exceo
        Raise Exception.Create(sErro);
      End
      Else
      Begin
        dm.cdsArquivo.Edit;
        // agora sim, gravo o nome do arquivo destino corretamente
        dm.cdsArquivoNOME_ARQUIVO.AsString := ExtractFileName(sDestino);
        dm.cdsArquivo.ApplyUpdates(0);
      End;
      // apago a linha do ListBox
      lstArquivos.Items.Delete(0);
    Except
      On E : Exception Do
      Begin
        // apago a linha do ListBox
        lstArquivos.Items.Delete(0);
        // qtd de erros ocorridos
        Inc(iTotErro);
        // cancela a gravao
        dm.cdsArquivo.CancelUpdates;
        sTXTErro := sTXTErro + sOrigem + ' - ' + e.Message +#13+#10;
      End;
    End;
  Until (lstArquivos.Count = 0);
  If iTotErro > 0 Then
  Begin
    // se ocorreram erros...
    MessageDlg(Format('%d arquivo(s) no foram importados.',[iTotErro])+#13+#10+
               sTXTErro, mtWarning, [mbOK], 0);
  End
  Else
  Begin
    // tudo certo
    MessageDlg('Arquivo(s) importados com sucesso!', mtInformation, [mbOK], 0);
  End;
end;

procedure TfrmCadArquivo.memObsKeyPress(Sender: TObject; var Key: Char);
begin
  bbObs.Enabled := True;
end;

procedure TfrmCadArquivo.Status;
// status do DataSource
begin
  // se o cdsArquivo estiver vazio...
  If dsArqruivo.DataSet.IsEmpty Then
  Begin
    // ... desabilita todos os botes
    bbAbrir.Enabled       := False;
    bbDelTabela.Enabled   := False;
    bbSalvar.Enabled      := False;
    bbSalvarTodos.Enabled := False;
  End
  Else
  Begin
    // hablita os botes
    bbAbrir.Enabled       := True;
    bbDelTabela.Enabled   := True;
    bbSalvar.Enabled      := True;
    bbSalvarTodos.Enabled := True;
    // se o cdsArquivo estiver em modo de edio...
    If dsArqruivo.DataSet.State = dsEdit Then
    Begin
      // ... hablita o boto para gravar a observao
      bbObs.Enabled := True;
    End
    Else
    Begin
      // ... desablita o boto para gravar a observao
      bbObs.Enabled := False;
    End;
  End;
end;

Procedure TfrmCadArquivo.Copia(AOrigem, ADestino : String);
// copia o arquivo localmente
begin
  If not FileExists(AOrigem) Then
  // se o arquivo no existir
  Begin
    Raise Exception.Create('O arquivo no existe no sistema.');
  End
  Else If not CopyFile(PChar(AOrigem),PChar(ADestino),false) Then
  // se der erro na cpia
  Begin
    Raise Exception.Create('Falha ao copiar o arquivo.');
  End;
end;

end.
