unit uRealizaPagamento;

interface

uses
  uPagamento, uGerente, uCaixa, uFrenteLoja, Messages, Dialogs;

type
  TRealizaPagamento = class
  private
    frenteLoja : TFrenteLoja;
    caixa : TCaixa;
    gerente : TGerente;

    procedure configurarObjetos;
    procedure executarPagamento(aValor : double);

    constructor Create;
    destructor Destroy;
  public
    function realizarPagamento(aValor : double) : Boolean;

    class function getInstance : TRealizaPagamento;
    class procedure destroyInstance;
  end;

implementation

var
  gRealizaPagamento : TRealizaPagamento = nil;

{ TRealizaPagamento }

procedure TRealizaPagamento.configurarObjetos;
begin
  frenteLoja.setSucessor(caixa);
  caixa.setSucessor(gerente);
end;

constructor TRealizaPagamento.Create;
begin
  frenteLoja := TFrenteLoja.Create;
  caixa := TCaixa.Create;
  gerente := TGerente.Create;
end;

destructor TRealizaPagamento.Destroy;
begin
  frenteLoja.Destroy;
  caixa.Destroy;
  gerente.Destroy;
end;

class procedure TRealizaPagamento.destroyInstance;
begin
  if Assigned(gRealizaPagamento) then begin
    gRealizaPagamento.Destroy;
  end;
end;

procedure TRealizaPagamento.executarPagamento(aValor: double);
begin
  MessageDlg(frenteLoja.processarPagamento(aValor), mtInformation, [mbOK], 0);
end;

class function TRealizaPagamento.getInstance: TRealizaPagamento;
begin
  if not Assigned(gRealizaPagamento) then begin
    gRealizaPagamento := TRealizaPagamento.Create;
  end;

  Result := gRealizaPagamento;
end;

function TRealizaPagamento.realizarPagamento(aValor : double): Boolean;
begin
  Result := False;
  try
    Self.configurarObjetos;
    Self.executarPagamento(aValor);

    Result := True;
  except
    Exit;
  end;
end;

end.
